@@ -1,7 +1,7 @@
 package Complete::Util;
 
 our $DATE = '2016-01-05'; # DATE
-our $VERSION = '0.44'; # VERSION
+our $VERSION = '0.45'; # VERSION
 
 use 5.010001;
 use strict;
@@ -107,6 +107,8 @@ sub __min(@) {
     $m;
 }
 
+our $code_editdist;
+
 # straight copy of Wikipedia's "Levenshtein Distance"
 sub __editdist {
     my @a = split //, shift;
@@ -182,8 +184,6 @@ _
     },
 };
 sub complete_array_elem {
-    state $code_editdist;
-
     my %args  = @_;
 
     my $array     = $args{array} or die "Please specify array";
@@ -192,7 +192,7 @@ sub complete_array_elem {
     my $ci          = $Complete::Common::OPT_CI;
     my $map_case    = $Complete::Common::OPT_MAP_CASE;
     my $word_mode   = $Complete::Common::OPT_WORD_MODE;
-    my $char_mode   = $Complete::Common::OPT_CHAR_MODE // 1;
+    my $char_mode   = $Complete::Common::OPT_CHAR_MODE;
     my $fuzzy       = $Complete::Common::OPT_FUZZY;
 
     return [] unless @$array;
@@ -295,7 +295,12 @@ sub complete_array_elem {
     # fuzzy matching
     if ($fuzzy && !@words) {
         $code_editdist //= do {
-            if (eval { require Text::Levenshtein::XS; 1 }) {
+            if (($ENV{COMPLETE_UTIL_LEVENSHTEIN} // '') eq 'xs') {
+                require Text::Levenshtein::XS;
+                \&Text::Levenshtein::XS::distance;
+            } elsif (($ENV{COMPLETE_UTIL_LEVENSHTEIN} // '') eq 'pp') {
+                \&__editdist;
+            } elsif (eval { require Text::Levenshtein::XS; 1 }) {
                 \&Text::Levenshtein::XS::distance;
             } else {
                 \&__editdist;
@@ -496,7 +501,7 @@ Complete::Util - General completion routine
 
 =head1 VERSION
 
-This document describes version 0.44 of Complete::Util (from Perl distribution Complete-Util), released on 2016-01-05.
+This document describes version 0.45 of Complete::Util (from Perl distribution Complete-Util), released on 2016-01-05.
 
 =head1 DESCRIPTION
 
@@ -648,6 +653,15 @@ Metadata (extra keys) for the hash.
 
 Return value:  (hash)
 
+=head1 ENVIRONMENT
+
+=head2 COMPLETE_UTIL_LEVENSHTEIN => str ('pp'|'xs')
+
+Can be used to force which levenshtein implementation to use. The default is to
+use XS version from L<Text::Levenshtein::XS> if that module is installed,
+otherwise fallback to the included PP implementation (which is about 1-2 orders
+of magnitude slower).
+
 =head1 SEE ALSO
 
 L<Complete>
@@ -657,6 +671,8 @@ L<Complete::Bash> or L<Getopt::Long::Complete> or L<Perinci::CmdLine>.
 
 Other C<Complete::*> modules.
 
+L<Bencher::Scenarios::CompleteUtil>
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Complete-Util>.
